#include <linux/module.h>
#include <linux/input.h>
#include <linux/timer.h>

static void ff_timer(unsigned long);

static struct input_dev *idev;
static DEFINE_TIMER(tim, ff_timer, 0, 0);

static void ff_timer(unsigned long data)
{
//	input_sync(idev);
	input_event(idev, EV_SYN, SYN_CONFIG, 0);
/*	printk(KERN_INFO "ff-test: timer\n");*/
	mod_timer(&tim, jiffies + 1); //msecs_to_jiffies(500));
}

static int ff_open(struct input_dev *idev)
{
	mod_timer(&tim, jiffies + 1);

	printk(KERN_INFO "ff-test: timer started\n");

	return 0;
}

static void ff_close(struct input_dev *idev)
{
	del_timer_sync(&tim);
	printk(KERN_INFO "ff-test: timer stopped\n");
}

static int ff_play(struct input_dev *idev, void *data, struct ff_effect *eff)
{
	struct timespec t;
	unsigned int a;

	getnstimeofday(&t);
	printk(KERN_INFO "ff-test: play effect [");
	for (a = 0; a < 6; a++)
		printk("%u ", eff->u.ff_raw[a]);
	printk("] %ld\n", t.tv_nsec/1000);

	return 0;
}

static int ff_init(void)
{
	int retval = -ENOMEM;

	idev = input_allocate_device();
	if (idev == NULL) {
		goto err;
	}

	idev->name = "FF test";
	idev->id.bustype = BUS_VIRTUAL;
	idev->id.vendor = 0x1234;
	idev->id.product = 0x4321;
	idev->id.version = 1;
	idev->open = ff_open;
	idev->close = ff_close;
	set_bit(FF_RAW, idev->ffbit);

	retval = input_ff_create_memless(idev, NULL, ff_play);
	if (retval) {
		goto err_frin;
	}

	retval = input_register_device(idev);
	if (retval) {
		goto err_frin;
	}

	printk(KERN_INFO "ff-test successfully loaded\n");

	return 0;
err_frin:
	input_free_device(idev);
err:
	return retval;
}

static void ff_exit(void)
{
	input_unregister_device(idev);
}

module_init(ff_init);
module_exit(ff_exit);

MODULE_LICENSE("GPL");
