#define _GNU_SOURCE
#include <fcntl.h>
#include <stdio.h>
#include <poll.h>
#include <signal.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "fpio.h"

static unsigned int comp = 1;

void sig(int sig)
{
	switch (sig) {
	case SIGTERM:
		comp = 0;
		break;
	default:
		fprintf(stderr, "Unhandled signal!\n");
	}
}

void compute(struct fp_compute *fc)
{
//	printf("j: %lu\n", fc->stamp);
	switch (fc->op) {
	case '*':
		fc->res = fc->x * fc->y;
		break;
	case '+':
		fc->res = fc->x + fc->y;
		break;
	default:
		fc->res = 0.0;
		fprintf(stderr, "Unsupported op '%c'\n", fc->op);
		break;
	}
}

int main(void)
{
	struct fp_stcom *fsc;
	struct pollfd fds = { .events = POLLIN };
	int retval = 1, a;
	__u32 cnt;

	fds.fd = open("fp", O_RDWR); /* fp is character device, see mknod(1) */
	if (fds.fd < 0) {
		perror("open");
		goto end;
	}

	if (ioctl(fds.fd, FPIOC_BUFCNT, &cnt) < 0) {
		perror("ioctl BUFCNT failed");
		goto cl;
	}

	fsc = mmap(0, sizeof *fsc + cnt * sizeof(struct fp_compute),
		PROT_READ | PROT_WRITE, MAP_SHARED, fds.fd, 0);
	if (fsc == MAP_FAILED) {
		perror("mmap failed");
		goto cl;
	}

	printf("Allocated %u buffers, overall size: %zu\n", cnt,
		sizeof *fsc + cnt * sizeof(struct fp_compute));

	signal(SIGTERM, sig);

	nice(-20);

	while (comp) {
		a = poll(&fds, 1, -1);
		if (a < 0) {
			perror("poll failed");
			goto unm;
		} else if (a == 0)
			continue; /* check if a signal was caught */

		if (fds.revents & (POLLERR|POLLHUP|POLLNVAL)) {
			fprintf(stderr, "Poll returned unexpected value for "
					"fd%d (0x%x)\n", fds.fd, fds.revents);
			break;
		}
#if 1
		if (fsc->wrptr >= cnt)
			fprintf(stderr, "Bad wrptr\n");
#endif

		printf("We are about to compute: %u/%u = %u\n", fsc->rdptr,
				fsc->wrptr, cnt - fsc->rdptr + fsc->rdptr);
		while (fsc->rdptr != fsc->wrptr) {
			compute(&fsc->c[fsc->rdptr]);

			fsc->rdptr = (fsc->rdptr + 1) % cnt;
		}
		printf("We are done: %u/%u\n", fsc->rdptr, fsc->wrptr);
		ioctl(fds.fd, FPIOC_DONE);
	}

	puts("Interrupted!");

	retval = 0;
unm:
	munmap(fsc, sizeof *fsc + cnt * sizeof(struct fp_compute));
cl:
	close(fds.fd);
end:
	return retval;
}
