#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "libphantom.h"

#define F_HIST_SIZE 9

#define SWAP(x, y) do { float tmp; tmp = x; x = y; y = tmp; } while (0)

static const float tbl[F_HIST_SIZE] = {
	1.5/10.0, -1/10.0, 2/10.0, 5/10.0, 2/10.0, -1/10.0, 1.5/10.0,
	0
};

/**
 * median_filter - implementation of unoptimized MEDIAN filter
 *
 * @out: filtered result
 * @in: input to be filtered
 */
static void median_filter(float out[3], float in[3])
{
	static float f_hist[F_HIST_SIZE][3];
	static float histsort[F_HIST_SIZE][3];
	static unsigned int hist = 0;
	unsigned int i;
	float x, y, z;

	i = 0; x = y = z = 0;
	memcpy(f_hist[hist], in, sizeof(in));

  	while (i != F_HIST_SIZE) {
		unsigned int idx = abs((hist - i) % F_HIST_SIZE);
		x += f_hist[idx][0] * tbl[i];
		y += f_hist[idx][1] * tbl[i];
		z += f_hist[idx][2] * tbl[i];
		i++;
	}

	for (i = 0; i < F_HIST_SIZE; i++) {
		unsigned int j, idx = abs((hist - i) % F_HIST_SIZE);
		float t = f_hist[idx][0];

		for (j = 0; j < i; j++)
			if (histsort[j][0] < t)
				SWAP(histsort[j][0], t);
		histsort[i][0] = t;

		t = f_hist[idx][1];
		for (j = 0; j < i; j++)
			if (histsort[j][1] < t)
				SWAP(histsort[j][1], t);
		histsort[i][1] = t;

		t = f_hist[idx][2];
		for (j = 0; j < i; j++)
			if (histsort[j][2] < t)
				SWAP(histsort[j][2], t);
		histsort[i][2] = t;
	}

	hist = (hist + 1) % F_HIST_SIZE;
	memcpy(out, histsort[F_HIST_SIZE/2], sizeof(out));

#if 0
	p->force[0] = x;
	p->force[1] = y;
	p->force[2] = z;
	usleep(10);
	printf("Forces: %f %f %f \r", p->force[0], p->force[1], p->force[2]);
#endif
}

void phm_filter_force(float force_out[3], float force_in[3])
{
	median_filter(force_out, force_in);
}

void phm_filter_1D_recursive(float force_out[3], float alpha, float force_in[3],
		float force_old[3])
{
	unsigned int i;

	for (i = 0; i < 3; i++)
		force_out[i] = alpha * force_in[i] +
				(1.0 - alpha) * force_old[i];

	memcpy(force_old, force_out, sizeof(float) * 3);
}

void phm_filter_init(void)
{
}
