#include <math.h>

#include "libphantom.h"

void phm_get_stylus_matrix(float *m,
		float a, float b, float y, float x, float z)
{
	float sin_a = sin(a);
	float sin_b = sin(b);
	float sin_x = sin(x);
	float sin_y = sin(y);
	float sin_z = sin(z);
	float cos_a = cos(a);
	float cos_b = cos(b);
	float cos_x = cos(x);
	float cos_y = cos(y);
	float cos_z = cos(z);

	float cos_a_cos_y = cos_a * cos_y;
	float sin_a_cos_b = sin_a * cos_b;

	float sin_a_cos_b_sin_y = sin_a_cos_b * sin_y;
	float sin_a_sin_b_cos_x = sin_a * sin_b * cos_x;
	float sin_a_cos_y = sin_a * cos_y;
	float sin_a_sin_b_sin_x = sin_a * sin_b * sin_x;

	float sin_b_sin_y = sin_b * sin_y;
	float sin_b_sin_y_cos_z = sin_b_sin_y * cos_z;
	float sin_b_sin_y_sin_z = sin_b_sin_y * sin_z;

	float sin_b_cos_y = sin_b * cos_y;
	float sin_b_cos_y_sin_x = sin_b_cos_y * sin_x;
	float sin_b_cos_y_cos_x = sin_b_cos_y * cos_x;

	float cos_a_cos_b_sin_y = cos_a * cos_b * sin_y;
	float cos_a_sin_b = cos_a * sin_b;
	float cos_a_sin_b_cos_x = cos_a_sin_b * cos_x;

	float cos_b_cos_x = cos_b * cos_x;

	float cos_b_sin_x = cos_b * sin_x;
	float cos_a_sin_b_sin_x = cos_a * sin_b * sin_x;

	float exp1 = cos_a * sin_y + sin_a * cos_b * cos_y;
	float exp1_sin_x = exp1 * sin_x;

	float exp2 = -sin_a * sin_y + cos_a * cos_b * cos_y;
	float exp2_sin_x = exp2 * sin_x;

	/* Multiplicated  matrix A x B x Y x X x Z */

	m[0] = (cos_a_cos_y - sin_a_cos_b_sin_y) * cos_z -
			(-sin_a_sin_b_cos_x - exp1_sin_x) * sin_z;

	m[1] = (cos_a_cos_y - sin_a_cos_b_sin_y) * sin_z +
			(-sin_a_sin_b_cos_x - exp1_sin_x) * cos_z;

	m[2] = -sin_a_sin_b_sin_x + exp1 * cos_x;

	m[3 + 0] = -sin_b_sin_y_cos_z -
			(cos_b_cos_x - sin_b_cos_y_sin_x) * sin_z;
	m[3 + 1] = -sin_b_sin_y_sin_z +
			(cos_b_cos_x - sin_b_cos_y_sin_x) * cos_z;
	m[3 + 2] = cos_b_sin_x + sin_b_cos_y_cos_x;

	m[6 + 0] = (-sin_a_cos_y - cos_a_cos_b_sin_y) * cos_z -
			(-cos_a_sin_b_cos_x - exp2_sin_x) * sin_z;
	m[6 + 1] = (-sin_a_cos_y - cos_a_cos_b_sin_y) * sin_z +
			(-cos_a_sin_b_cos_x - exp2_sin_x) * cos_z;
	m[6 + 2] = -cos_a_sin_b_sin_x + exp2 * cos_x;
}

void phm_get_stylus_vector(float *vector_out,
		const float *matrix, const float *vector_in)
{
	unsigned int i;

	for (i = 0; i < 3; i++) {
		vector_out[i] = matrix[i * 3] * vector_in[0] +
				matrix[i * 3 + 1] * vector_in[1] +
				matrix[i * 3 + 2] * vector_in[2];
	}
}
