//-----------------------------------------------------------//
//							     //
//       mr_thread.h	       ver. 3.0			     //
//							     //
//       abstract class for threads -- it provides	     //
//       basic skeleton for work with threads -- like	     //
//       creation of thread and all other necessary things   //
//							     //
//       Pavel Kolcarek, kolcarek@fi.muni.cz		     //
//							     //
//-----------------------------------------------------------//

#ifndef __MR_THREAD_H__
#define __MR_THREAD_H__

#include "gm.h"
#include <pthread.h>

  ////////////////////////////////////////////////////////////
 //
//   class CThread
//   --------------------------------------------------------
//   abstract thread class 

class CThread {
protected:
	// the identifier of newly created phantom thread
	pthread_t thread_id;

	// mutex lock -- it is being used
	// during all computations -- any method which want
	// to change any attribute have to obtain this lock.
	pthread_mutex_t thread_lock;

	// after setting this flag, thread loop will
	// quit (after finishing current iteration)
	bool wanna_quit_loop;

	// state of thread loop -- true if is it running
	// otherwise false
	bool loop_is_running;

	// how does it all works:
	// --------------------------------------------------
	//
	// * the constructor CThread() is called at first.
	//   it will initialize mutex lock.
	//   
	// * then we'll call start_loop() function, which
	//   will create thread with 'entry point' function
	//   init_thread()
	//   
	// * in init_thread() we will typecast all necesary
	//   data and pass them to the run_thread() function
	//
	// * and here we go -- run_thread() is a loop
	//   function. put here all code you want thread to do.

	// this is a main thread function -- all 'user' code
	// goes here
	virtual void run_thread();

	// entry thread function can't be a normal member
	// function of a class. however, it can be a static
	// member function of a class.
	static void *init_thread(void *);

	// methods for locking and unlocking thread.
	// they are used during all computations 
	void lock_me();
	void unlock_me();

public:
	// constructor & destructor
	CThread();
	virtual ~ CThread();

	// this function create thread. on success
	// a 0 is returned, on error a non-zero error code
	// is returned
	int start_loop();

	// find out if this thread is running
	// returns true -- loop is running, otherwise
	// return false
	bool is_running();
};

#endif
