//-----------------------------------------------------------//
//							     //
//       ph_lib_data_structs.cpp	      ver. 3.0       //
//							     //
//       implementation of CPhantomRec and PhTriangle	     //
//       classes. Their descriptions are in the header file. //
//							     //
//       Pavel Kolcarek, kolcarek@fi.muni.cz		     //
//							     //
//-----------------------------------------------------------//

#include <stdlib.h>
#include <unistd.h>
#include "ph_lib_data_structs.h"

  ////////////////////////////////////////////////////////////
 //
//   class CPhTriangle

CPhTriangle::CPhTriangle(gmVector3 & v1, gmVector3 & v2, gmVector3 & v3, gmVector3 & n, gmVector3 & vn1, gmVector3 & vn2, gmVector3 & vn3):
active(false), has_real_normal(false)
{
	vertex[0] = v1;
	vertex[1] = v2;
	vertex[2] = v3;
	normal = n;
	vertex_normal[0] = vn1;
	vertex_normal[1] = vn2;
	vertex_normal[2] = vn3;
}

  ////////////////////////////////////////////////////////////
 //
//   class CPhantomRec

CPhTriangleSet::CPhTriangleSet():count(0),
is_ready(true), triangles(PH_MAX_TRIANGLES)
{
	for (int i = 0; i < PH_MAX_TRIANGLES; i++) {
		triangles[i] = new CPhTriangle();
	}
}

CPhTriangleSet::~CPhTriangleSet()
{
	triangles.clear();
}

  ////////////////////////////////////////////////////////////
 //
//   class CPhantomRec

CPhantomRec::CPhantomRec():ph_dev_position(),
active_set(0),
ph_force_radius(PH_FORCE_RADIUS),
ph_min_radius(PH_MIN_RADIUS),
ph_max_force(PH_MAX_FORCE),
ph_force_delta(PH_FORCE_DELTA),
ph_force_delta_virtual(PH_FORCE_DELTA_VIRTUAL), ph_attrition(false)
{
	triangle_set[0] = new CPhTriangleSet();
	triangle_set[1] = new CPhTriangleSet();
}

CPhantomRec::~CPhantomRec()
{
	delete triangle_set[0];
	delete triangle_set[1];
}

// get and update actual speed, position and force from
// phantom internal structure
void CPhantomRec::update_phantom_info(struct phantom_info *phantom_rec)
{
#ifdef PHANTOM_PRESENT
	ph_dev_position[0] = phantom_rec->actual.pos[0];
	ph_dev_position[1] = phantom_rec->actual.pos[1];
	ph_dev_position[2] = phantom_rec->actual.pos[2];

	// rotation and button state of stylus
	const float e[3][3] = {
		{1, 0, 0},
		{0, 1, 0},
		{0, 0, 1}
	};

	float ph_m[9] = { 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0 };
	float v[3][3] = { {0.0, 0.0, 0.0}, {0.0, 0.0, 0.0}, {0.0, 0.0, 0.0} };

	phm_get_stylus_matrix(ph_m,
			      phantom_rec->actual.alpha,
			      phantom_rec->actual.beta,
			      phantom_rec->actual.pos_enc[0],
			      phantom_rec->actual.pos_enc[1],
			      phantom_rec->actual.pos_enc[2]);

	for (int i = 0; i < 3; i++) {
		phm_get_stylus_vector(v[i], ph_m, e[i]);
	}

	ph_stylus_direction = gmVector3(v[2][0], v[2][1], v[2][2]);
	ph_stylus_rotate = gmMatrix4(ph_m[0], ph_m[3], ph_m[6], 0.0,
				     ph_m[1], ph_m[4], ph_m[7], 0.0,
				     ph_m[2], ph_m[5], ph_m[8], 0.0,
				     0.0, 0.0, 0.0, 1.0);

	ph_button_state = phm_button_state(ph_global);
	// --------------------------------------

	ph_speed[0] = phantom_rec->actual.speed[0];
	ph_speed[1] = phantom_rec->actual.speed[1];
	ph_speed[2] = phantom_rec->actual.speed[2];
#else
	ph_dev_position[0] = ph_dev_position[1] = ph_dev_position[2] = 0.0;
	ph_speed[0] = ph_speed[1] = ph_speed[2] = 0.0;
#endif
}

// we want to update only force, set it to phantom internal structure
void CPhantomRec::set_phantom_info(struct phantom_info *phantom_rec)
{
#ifdef PHANTOM_PRESENT
	phantom_rec->force[0] = ph_force[0];
	phantom_rec->force[1] = ph_force[1];
	phantom_rec->force[2] = ph_force[2];
#endif
}

CPhTriangleSet *CPhantomRec::get_active_set()
{
	assert(triangle_set[active_set] != NULL);

	return triangle_set[active_set];
}

CPhTriangleSet *CPhantomRec::get_non_active_set()
{
	assert(triangle_set[1 - active_set] != NULL);

	return triangle_set[1 - active_set];
}

bool CPhantomRec::switch_sets()
{
	// if non-active set is ready to use
	if (triangle_set[1 - active_set]->is_ready) {
		// mark the active set as NOT READY
		triangle_set[active_set]->is_ready = false;
		// and change pointer to the active set to the second one
		active_set = 1 - active_set;

		return true;	// switch was succesful
	} else {
		return false;
	}
}
