//-----------------------------------------------------------//
//							     //
//       ph_lib_def.h		 ver. 3.0		     //
//							     //
//       Various constants...				     //
//       any of them can be used in different file,	     //
//       so I've put them into one header file here.	     //
//							     //
//       Pavel Kolcarek, kolcarek@fi.muni.cz		     //
//							     //
//-----------------------------------------------------------//

#ifndef __PH_LIB_DEF_H__
#define __PH_LIB_DEF_H__

#define NEXT3(i) (((i) + 1) % 3)

#define PHANTOM_PRESENT		// if is PHANTOM_PRESENT defined
				// then will be used whole code.
				// if not, we turn off all
				// "hardware parts" of code, so
				// it can run indipendently

// min/max coordinates of PHANToM working space
// NOTE: PhTextur uses different values
// NOTE: Values obtained from PHANToM:
//       min: ( -0.260157, -0.113381, -0.13774)
//       max: (  0.257271 , 0.395307,  0.118629)
#define PH_MIN_X -0.260
#define PH_MAX_X  0.257

#define PH_MIN_Y -0.113
#define PH_MAX_Y  0.395

#define PH_MIN_Z -0.138
#define PH_MAX_Z  0.119

// center of the PHANToM workspace
#define PH_CENTER_X (PH_MIN_X + PH_MAX_X) / 2.0
#define PH_CENTER_Y (PH_MIN_Y + PH_MAX_Y) / 2.0
#define PH_CENTER_Z (PH_MIN_Z + PH_MAX_Z) / 2.0

#define DEFAULT_RATE		2000	// initial phantom rate
#define DEFAULT_MAX_FORCE	5
#define DEFAULT_MAX_SPEED	0.6	// max. speed of the PHANToM probe

// other constants from PhTextur:

#define PH_MAX_TRIANGLES	64
#define PH_MODEL_MAX_RADIUS	4.0

// phtextur -> mr scale factor: 17200
// phtextur: #define PH_FORCE_RADIUS 100	// 200 ?
//#define PH_FORCE_RADIUS 0.00581
#define PH_FORCE_RADIUS		0.00581
//#define PH_MIN_RADIUS		0.0001744
#define PH_MIN_RADIUS		0.0001
#define PH_FORCE_DELTA		400.23	// 0.1 ?
#define PH_FORCE_DELTA_VIRTUAL	60.2
#define PH_FORCE_DELTA_ATTRITION 2
#define PH_FORCE_MAX_ATTRITION	5
#define PH_MAX_FORCE		40

#endif
