//-----------------------------------------------------------//
//							     //
//       ph_lib_force.cpp		    ver. 3.2	     //
//							     //
//       Implementation of CPhantomForce class.		     //
//							     //
//       Pavel Kolcarek, kolcarek@fi.muni.cz		     //
//							     //
//-----------------------------------------------------------//

#include "ph_lib_force.h"

  ////////////////////////////////////////////////////////////
 //
//   class CPhantomForce

void CPhantomForce::init_attributes()
{
	attributes_initialized = true;

	old_device_position = phantom_rec.get_device_position();
	virtual_position = phantom_rec.get_device_position();
}

void CPhantomForce::move_virtual_position(CPhTriangleSet * data,
					  gmVector3 & vector)
{
	int i, ti;
	double min_coef;
	double coef1, coef2;
	CPhTriangle *min_triangle;
	CPhTriangle *triangle;
	gmVector3 vec1;
	gmVector3 vec2;
	gmVector3 point_on_plane;

	min_coef = 2.0;
	min_triangle = NULL;

	// for each triangle of the active set
	for (ti = 0; ti < data->count; ti++) {
		triangle = data->triangles[ti];

		// find out if line given by virtual_position and vector
		// intersects the plane (given by vertex[0] and normal)
		coef1 = plane_intersect(virtual_position, vector,
					triangle->vertex[0], triangle->normal);

		// if such an intersection exists (coef <= 1 & > 0)
		if (coef1 > 0.0 && coef1 <= 1.0 && coef1 < min_coef) {
			point_on_plane = virtual_position + vector * coef1;

			// asi to zjistuje, jestli point_on_plane lezi uvnitr
			// (pak se ulozi min_triangle a min_coef) nebo vne
			// trojuh.
			for (i = 0; i < 3; i++) {
				vec1 = triangle->vertex[NEXT3(i)] -
					triangle->vertex[i];
				vec2 = cross(triangle->normal, vec1);
				coef2 = plane_distance(point_on_plane,
						   triangle->vertex[i], vec2);

				if (coef2 > 0.0)
					goto continue2;
			}

			min_triangle = triangle;
			min_coef = coef1;
		}
continue2:;
	}

	// if no intersection along vector (= between ph_position and 
	// ph_old_position) was found
	if (min_coef == 2.0) {
		virtual_position += vector;
	} else {
		vec1 = min_triangle->normal * phantom_rec.get_min_radius() +
		    min_triangle->vertex[0];

		min_coef = plane_intersect(virtual_position, vector,
					   vec1, min_triangle->normal);

		if (min_coef > 0.0)
			virtual_position = virtual_position + vector * min_coef;
	}
}

void CPhantomForce::determine_virtual_position(CPhTriangleSet * data)
{
	gmVector3 vector;

	// compute vector between new and old position of the PHANToM
	// and if the new position is inside the object, move virtual 
	// position to the surface of the object
	vector = phantom_rec.get_device_position() - old_device_position;
	move_virtual_position(data, vector);

	// check it once again
	vector = phantom_rec.get_device_position() - virtual_position;
	// if the vector is zero, everything is OK, return
	if (vector.isZero())
		return;

	/* TODO */// zkusit to tady zrusit, pac presne nevim co to
	// dela a jestli je to vubec nutne...
	// otherwise move virtual position on the surface
	move_virtual_position(data, vector);
}

int CPhantomForce::get_real_position(CPhTriangleSet * data,
				     CPhTriangle * triangle,
				     gmVector3 & pos_in_trian)
{
	int i;
	double dist[3];
	double dist_coef;
	gmVector3 v;
	gmVector3 R;
	gmVector3 edge;
	gmVector3 norm[3];
	gmVector3 final_norm;

	dist_coef = plane_distance(virtual_position,
				   triangle->vertex[0], triangle->normal);

	// if the virtual position is out of this triangle, don't continue 
	if (-dist_coef > phantom_rec.get_force_radius() || dist_coef >= 0) {
		return -1;
	}

	for (i = 0; i < 3; i++) {	// for each edge of this triangle

		edge = triangle->vertex[NEXT3(i)] - triangle->vertex[i];

		if (edge.isZero()) {
			v = cross(triangle->vertex_normal[i],
				  triangle->vertex_normal[NEXT3(i)]);
			dist_coef = plane_distance(virtual_position,
					   triangle->vertex[i], v);

			if (dist_coef > 0)
				return -2;

			dist[i] = 1;
			norm[i].assign(0, 0, 0);

			continue;

		} else {
			// find the nearest point (dist_coef for the edge) from
			// edge to the virtual position (dist_coef ~ B )
			dist_coef = plane_distance(triangle->vertex[i],
					   virtual_position, edge);

			if (dist_coef < 0) {
				dist_coef = 0;
			} else if (dist_coef > 1) {
				dist_coef = 1;
			}
			// compute the intersection R from virtual position to
			// the plane (vertex[i] & edge[i]), i.e. to the nearest
			// point
			/* R = edge * dist_coef + vertex[0] */
			R = edge * dist_coef + triangle->vertex[i];
		}

		// in the nearest point compute new normal norm[i] by the
		// interpolation of the normals normal[i] & normal[i+1]
		/* v = (norm[1] - norm[0]) * d + norm[0] */
		norm[i] = triangle->vertex_normal[NEXT3(i)] -
			triangle->vertex_normal[i];
		norm[i] = norm[i] * dist_coef + triangle->vertex_normal[i];
		norm[i].normalize();

		v = cross(norm[i], edge);

		v.normalize();

		// compute the distance from intersection R to the virtual
		// position
		dist[i] = plane_distance(virtual_position, R, v);
		// if virtual position is out of this triangle, don't continue
		if (dist[i] > 0)
			return -3;
	}

//	if (dist[0] <= 0 && dist[1] <= 0 && dist[2] <= 0) {
	if (dist[0] != 0) {
		norm[0] /= -dist[0];

		if (dist[1] != 0) {
			norm[1] /= -dist[1];

			if (dist[2] != 0) {
				norm[2] /= -dist[2];
				final_norm = norm[0] + norm[1] + norm[2];
			} else
				final_norm = norm[2];
		} else
			final_norm = norm[1];
	} else
		final_norm = norm[0];

	final_norm.normalize();

	// find out intersection of this triangle and new interpolated
	// normal n = norm[0] + norm[1] + norm[2]
	dist_coef = plane_intersect(virtual_position, final_norm,
				    triangle->vertex[0], triangle->normal);
	// compute the final position in this triangle (this is a first part of
	// pos_in_trian =  virtual_position + final_norm * dist_coef )
	pos_in_trian = final_norm * dist_coef;

	if (pos_in_trian.length() > phantom_rec.get_force_radius())
		return -4;

	// finish the computing of the final position in this triangle
	pos_in_trian += virtual_position;

	return 0;
}

void CPhantomForce::update_force(gmVector3 & force,
				 gmVector3 & position_in_trian,
				 double &max_force)
{
	double length;
	gmVector3 force_vect;

	// distance from the virtual position to the position in the triangle
	force_vect = virtual_position - position_in_trian;

//	length = position_in_trian.length();
	length = force_vect.length();

	// if this distance is bigger than force radius, i.e. user's fingertip
	// is far away from the surface, quit the function
	if (length >= phantom_rec.get_force_radius())
		return;

	// cim vic se zaboruje, tim vic je vytlacovany
	length = (phantom_rec.get_force_radius() - length) *
	    phantom_rec.get_force_delta();
//	length = (phantom_rec.get_force_radius() - length) * 
//		(phantom_rec.get_force_radius() - length) * 
//		phantom_rec.get_force_delta();

//	position_in_trian.normalize();
//	position_in_trian /= -length;
//	force_vect *= -length;
	force_vect.normalize();
	force_vect *= length;

//	force += position_in_trian;
	force += force_vect;

	if (length > max_force)
		max_force = length;
}

gmVector3 CPhantomForce::compute_force(CPhTriangleSet * data, double &max_force)
{
	int result;	// return value of the get_real_position() function
	double length;

	gmVector3 force;	// a force exerted to the user
	gmVector3 position_in_trian;	// position of the phantom in the active triangle
	gmVector3 vector;

	max_force = 0;

	if (!attributes_initialized) {
		init_attributes();	// we call this method only once
		// after first call of update_phantom_info()
	}

	determine_virtual_position(data);

	// for each triangle of the active set of triangles do
	for (int i = 0; i < data->count; i++) {

		// if the user touches this triangle, find out the real touch
		// point within it
		result = get_real_position(data, data->triangles[i],
				position_in_trian);

		if (result == 0) {	// user touched this triangle

			// put down active flags
			data->triangles[i]->active_point = position_in_trian;
			data->triangles[i]->active = true;

			// vypocet sily s mapovanim textury, zatim vynechano
			//st_force_textur (data, &data->triangle[i],
			// position_in_trian); from virtual position and
			// position in the current triangle
			// compute partial force and add it to the total one
			update_force(force, position_in_trian, max_force);
		} else
			data->triangles[i]->active = false;

	}

	// the force extension in the virtual position
	vector = virtual_position - phantom_rec.get_device_position();
	vector *= phantom_rec.get_force_delta_virtual();
	force += vector;
//	force *= vector.length();

	length = vector.length();

	// remember maximal force
	if (length > max_force)
		max_force = length;

	return force;
}
