//-----------------------------------------------------------//
//							     //
//       ph_lib_force.h		     ver. 3.2		     //
//							     //
//       !!! TBS !!!					     //
//							     //
//       Pavel Kolcarek, kolcarek@fi.muni.cz		     //
//							     //
//-----------------------------------------------------------//

#ifndef __PH_LIB_FORCE_H__
#define __PH_LIB_FORCE_H__

#include "ph_lib_def.h"
#include "ph_lib_data_structs.h"

  ////////////////////////////////////////////////////////////
 //
//   class CPhantomForce
//   --------------------------------------------------------
//   !!! TBS !!!
//   

class CPhantomForce {
protected:
	// Phantom position, speed, actual force and other atributes.
	// Within this record we are using two sets: 
	// active -- used for computation of the force, and
	// non-active -- used for obtaining fresh data from 
	// multiresolution surface
	CPhantomRec phantom_rec;

	// real and virtual positions of the PHANToM
	gmVector3 virtual_position;
	gmVector3 old_virtual_position;
	gmVector3 old_device_position;

	// all attributes (virtual/device positions) are initialized
	// AFTER first call update_phantom_info()
	bool attributes_initialized;

	void init_attributes();

	// just determine virtual position ;-) This is main
	// method, move_virtual_position() is called within it.
	void determine_virtual_position(CPhTriangleSet *);

	// find out if new phantom position intersects the surface.
	// if yes, move the new position to the virtual one that
	// lies on the surface
	void move_virtual_position(CPhTriangleSet *, gmVector3 &);

	// find out the position where the user touches the triangle
	// if the position is found, 0 is returned
	int get_real_position(CPhTriangleSet *, CPhTriangle *, gmVector3 &);

	// from virtual position and position in the current triangle
	// compute partial force and add it to the total one
	void update_force(gmVector3 &, gmVector3 &, double &);

public:
	CPhantomForce():attributes_initialized(false) {};

	// find out current phantom position, speed and force.
	// these methods are used for other threads, e.g. model thread
	CPhantomRec *get_phantom_rec()	{ return &phantom_rec; };
	// or find it out separately   
	gmVector3 get_device_position()	{
		return phantom_rec.get_device_position();
	};
	gmVector3 get_position()	{ return phantom_rec.get_position(); };
	gmVector3 get_speed()		{ return phantom_rec.get_speed(); };

	/* !!!! TODO !!!! */// tohle je hlavni metoda, uvnitr se bude nejspis
	// jeste delit na vypocet pro troj./NURBS...
	// nebo jednodussi bude ji pretizit pro lib. ucel
	gmVector3 compute_force(CPhTriangleSet *, double &);

	/* TODO */
	// + milion pridruzenych metod pro god object, normalizaci, opravy...
	// + metody pro materialy
	// + metody pro vypocet normal/tangencial, moving vectors...
};

#endif
