//-----------------------------------------------------------//
//							     //
//       ph_lib_thread.h		ver. 3.0	     //
//							     //
//       description of each class can be found below	     //
//							     //
//       Pavel Kolcarek, kolcarek@fi.muni.cz		     //
//							     //
//-----------------------------------------------------------//

#ifndef __PH_LIB_THREAD_H__
#define __PH_LIB_THREAD_H__

#include <libphantom.h>

#include "gm.h"
#include "ph_lib_def.h"
#include "ph_lib_force.h"
#include "mr_thread.h"

#include <vector>

// forward declarations
class CModelThread;

  ////////////////////////////////////////////////////////////
 //
//   class CPhantomThread
//   --------------------------------------------------------
//   thread class -- here is done all haptics stuff:
//   initializing and closing phantom device, compute actual
//   force according to multiresolution model etc.
//   Data structures are defined in module ph_lib_data_structs
//   and force is computed in class CPhantomForce.
//   

class CPhantomThread:public CThread, public CPhantomForce {
private:
	// a maximal force which can be exerted on user
	double max_force;

	// a frequency of updating phantom_rec (it can be set
	// in the constructor only!!!)
	int phantom_rate;

	struct phantom_device *ph_global;

	// this is a main phantom thread function -- all
	// force computations etc. are done here
	virtual void run_thread();

	// before we send the final force to the PHANToM, we will
	// correct it a bit -- we'll apply filters like friction,
	// maximal force cropping, etc.
	void apply_filters(gmVector3 &);

	void (*apply_additional_filters)(gmVector3 &);

	// filters:
	//
	// add friction to the computed force
	void filter_add_friction(gmVector3 &);
	// crop the force according to max_force
	void filter_crop_force(gmVector3 &);

public:
	// constructor -- set attributes, open phantom
	// device, initialize mutex lock
	CPhantomThread(int);
	// destructor -- close phantom device, destroy mutex
	~CPhantomThread();

	// this is publicly accessable function, which create
	// a phantom thread
	// !!! TBS !!!
	void start_phantom_thread();
	void stop_phantom_thread();

	/* !!!! TODO !!!! */
	void enable_force();
	void disable_force();

	void set_additional_filters_func_ptr(void (*ptr)(gmVector3 &)) {
		apply_additional_filters = ptr;
	}
};

#endif
