#include <fcntl.h>
#include <poll.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <linux/types.h>

#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>

#include <phantom.h>

#define print2(args...) fprintf(stderr, args)

#ifndef DEBUG
# define DEBUG		0
#endif

int main(int argc, char **argv)
{
	struct phm_regs regs = { .mask = 0xff };
	struct phm_reg reg = { .value = ~0x1f };
	struct pollfd ufd;
	struct timeval tv;
	unsigned int i, oh = 0, omit_stop = 0;
	char *devname = "/dev/phantom0";
	int fd, tmp, retval = 1;
	int dir = 0;

	switch (argc) {
	case 3:
		omit_stop = 1;
	case 2:
		devname = argv[1];
	case 1:
		break;
	default:
		print2("Bad arguments count!\n");
		goto err;
	}

	printf("devname: %s, omit_stop=%u\n", devname, omit_stop);

	fd = open(devname, O_RDWR);
	if (fd < 0) {
		perror("open failed");
		goto err;
	}

	tmp = open(devname, O_RDWR);
	if (tmp >= 0) {
		print2("the second open didn't fail (and it should)!\n");
		close(tmp);
		goto err_close;
	}

	puts("Resetting the device");
	if (ioctl(fd, PHN_SETREG, &reg)) {
		perror("ioctl reset PHN_SETREG failed");
		goto err_close;
	}
	reg.value = 0x1f;
	if (ioctl(fd, PHN_SETREG, &reg)) {
		perror("ioctl reset PHN_SETREG failed");
		goto err_close;
	}

	puts("try to 1st poll");

	ufd.fd = fd;
	ufd.events = POLLIN | POLLERR | POLLNVAL;
	retval = poll(&ufd, 1, 0);
	if (retval < 0)
		perror("first poll failed");
	if (ufd.revents & POLLNVAL)
		print2("first poll failed - POLLNVAL\n");
	if (!(ufd.revents & POLLERR))
		print2("first poll failed - !POLLERR\n");

	if (retval < 0 || ufd.revents & POLLNVAL || !(ufd.revents & POLLERR)) {
		retval = 2;
		goto err_close;
	}

	retval = 3;
	puts("first poll successfull, trying to start");
	fflush(stdout);
	fflush(stderr);

	if (!oh) {
		if (ioctl(fd, PHN_NOT_OH) < 0) {
			perror("ioctl NOT_OH failed");
			goto err_close;
		}
	}

	reg.reg = PHN_CONTROL;
	reg.value = PHN_CTL_IRQ;
	if (ioctl(fd, PHN_SETREG, &reg) < 0) {
		perror("ioctl start failed");
		goto err_close;
	}

	if (ioctl(fd, PHN_NOT_OH) >= 0) {
		perror("ioctl NOT_OH didn't failed (and it should)");
		goto err_close;
	}

	puts("start successfull");

	gettimeofday(&tv, NULL);
	printf("time: %lf\n", (double)(tv.tv_sec % 100) +
			(double)tv.tv_usec / 1000000);

	for (i = 0; i < 20000; i++) {
		ufd.fd = fd;
		ufd.events = POLLIN | POLLERR | POLLNVAL;
		retval = poll(&ufd, 1, -1);
		if (retval < 0)
			perror("poll failed");
		if (ufd.revents & POLLNVAL)
			print2("poll failed - POLLNVAL\n");
		if (ufd.revents & POLLERR)
			print2("poll failed - POLLERR\n");

		if (retval < 0 || (ufd.revents & POLLNVAL) ||
				(ufd.revents & POLLERR)) {
			retval = 4;
			goto err_stop;
		}

		if (!(i % 250))
			dir ^= 1;
		regs.values[0] = (dir * 2 - 1) * 200 - 70 + PHN_ZERO_FORCE;
		regs.values[1] = regs.values[2] = PHN_ZERO_FORCE;
		regs.count = 3;
		if (ioctl(fd, PHN_SETREGS, &regs)) {
			perror("torques upload failed");
			goto err_stop;
		}
		if (oh) {
			reg.value ^= PHN_CTL_AMP;
			if (ioctl(fd, PHN_SETREG, &reg) < 0) {
				perror("ioctl amp failed");
				goto err_close;
			}
		}
		regs.count = 8;
		if (ioctl(fd, PHN_GETREGS, &regs))
			perror("ioctl get pos failed");
#if DEBUG == 0
		if ((i % 50) == 0) {
			putchar('.');
			fflush(stdout);
		}
#endif
#if DEBUG >= 2
		{
		unsigned int j;
		for (j = 0; j < 8; j++)
			printf("%.8lx ", regs.values[j]);
		puts("");
		}
#endif
#if DEBUG >= 1
		gettimeofday(&tv, NULL);
		printf("event: %x, time: %lf\n", ufd.revents,
				(double)(tv.tv_sec % 100) +
				(double)tv.tv_usec / 1000000.0);
#endif
	}
	
	puts("\npoll done");

	retval = 0;
err_stop:
	if (!omit_stop) {
		reg.reg = PHN_CONTROL;
		reg.value = 0;
		if (ioctl(fd, PHN_SETREG, &reg))
			perror("stop ioctl failed");
	}
err_close:
	close(fd);
err:
	return retval;
}
